/*!
 * \file HttpRequest.cs
 * \author Antoine PEREDERII
 * \brief Fichier contenant la classe HttpRequest.
 */

using System.Diagnostics;
using System.Net.Http.Json;
using Dto;
using Shared;

/*!
 * \brief Classe représentant un client HTTP pour les requêtes vers un service de gestion de livres.
 */
public class HttpRequest<T> where T : class
{
    private HttpClient _httpClient { get; } = new HttpClient();
    public HttpClient HttpClient => _httpClient;

    /*!
     * \brief Récupère tous les livres de manière asynchrone.
     * \return Une tâche représentant l'opération asynchrone qui retourne une liste de T.
     */
    public async Task<List<T>> GetAllAsync()
    {
        return await _httpClient.GetFromJsonAsync<List<T>>("");
    }

    /*!
     * \brief Récupère les activités par index et compte de manière asynchrone.
     * \param index L'index de départ pour la pagination.
     * \param count Le nombre d'éléments à récupérer.
     * \return Une tâche représentant l'opération asynchrone qui retourne une liste de T.
     */
    public async Task<List<T>> GetAsync((ActivityOrderCriteria, AthleteOrderCriteria) criteria, bool descending, int index, int count)
    {
        return await _httpClient.GetFromJsonAsync<List<T>>($"?OrderingPropertyName={criteria}&Descending={descending}&Index={index}&Count={count}");
    }

    /*!
     * \brief Récupère un livre par son identifiant de manière asynchrone.
     * \param id L'identifiant du livre à récupérer.
     * \return Une tâche représentant l'opération asynchrone qui retourne une liste de T.
     */
    public async Task<List<T>> GetByIdAsync(int id)
    {
        return await _httpClient.GetFromJsonAsync<List<T>>($"{id}");
    }

    public Task<int> GetNbItems()
    {
        return _httpClient.GetFromJsonAsync<int>("count");
    }

    public Task<IEnumerable<T>?> GetActivitiesByUser(int userId, int index, int count, (ActivityOrderCriteria, AthleteOrderCriteria) orderCriteria, bool descending = false)
    {
        return _httpClient.GetFromJsonAsync<IEnumerable<T>?>($"?userId={userId}&index={index}&count={count}&orderCriteria={orderCriteria}&descending={descending}");
    }

    public Task<int> GetNbActivitiesByUser(int userId)
    {
        return _httpClient.GetFromJsonAsync<int>($"count?userId={userId}");
    }

    /*!
     * \brief Ajoute un livre de manière asynchrone.
     * \param book Le livre à ajouter.
     * \return Une tâche représentant l'opération asynchrone qui retourne le livre ajouté (T).
     */
    public async Task<T> PostAsync(T activity)
    {
        var response = await _httpClient.PostAsJsonAsync("", activity);

        return await response.Content.ReadFromJsonAsync<T>();
    }

    /*!
     * \brief Met à jour un livre de manière asynchrone.
     * \param id L'identifiant du livre à mettre à jour.
     * \param book Les nouvelles données du livre à mettre à jour.
     * \return Une tâche représentant l'opération asynchrone qui retourne le livre mis à jour (T).
     */
    public async Task<T> PutAsync(int id, T activity)
    {
        var response = await _httpClient.PutAsJsonAsync($"{id}", activity);

        return await response.Content.ReadFromJsonAsync<T>();
    }

    /*!
     * \brief Supprime un livre de manière asynchrone.
     * \param id L'identifiant du livre à supprimer.
     * \return Une tâche représentant l'opération asynchrone.
     */
    public async Task DeleteAsync(int id)
    {
        await _httpClient.DeleteAsync($"{id}");
    }
}